@@ -1,5 +1,5 @@
 /* Language-level data type conversion for GNU C++.
-   Copyright (C) 1987-2024 Free Software Foundation, Inc.
+   Copyright (C) 1987-2026 Free Software Foundation, Inc.
    Hacked by Michael Tiemann (tiemann@cygnus.com)
 
 This file is part of GCC.
@@ -36,6 +36,7 @@ along with GCC; see the file COPYING3.  If not see
 #include "stringpool.h"
 #include "attribs.h"
 #include "escaped_string.h"
+#include "gcc-urlifier.h"
 
 static tree convert_to_pointer_force (tree, tree, tsubst_flags_t);
 static tree build_type_conversion (tree, tree);
@@ -574,6 +575,19 @@ force_rvalue (tree expr, tsubst_flags_t complain)
   return expr;
 }
 
+/* Force EXPR to be an lvalue, if it isn't already.  */
+
+tree
+force_lvalue (tree expr, tsubst_flags_t complain)
+{
+  if (!lvalue_p (expr))
+    {
+      expr = cp_build_addr_expr (expr, complain);
+      expr = cp_build_indirect_ref (input_location, expr, RO_ARROW, complain);
+    }
+  return expr;
+}
+
 
 /* If EXPR and ORIG are INTEGER_CSTs, return a version of EXPR that has
    TREE_OVERFLOW set only if it is set in ORIG.  Otherwise, return EXPR
@@ -608,11 +622,12 @@ cp_fold_convert (tree type, tree expr)
   tree conv;
   if (TREE_TYPE (expr) == type)
     conv = expr;
-  else if (TREE_CODE (expr) == PTRMEM_CST
-	   && same_type_p (TYPE_PTRMEM_CLASS_TYPE (type),
-			   PTRMEM_CST_CLASS (expr)))
+  else if ((TREE_CODE (expr) == PTRMEM_CST
+	    && same_type_p (TYPE_PTRMEM_CLASS_TYPE (type),
+			    PTRMEM_CST_CLASS (expr)))
+	    || (REFLECT_EXPR_P (expr) && REFLECTION_TYPE_P (type)))
     {
-      /* Avoid wrapping a PTRMEM_CST in NOP_EXPR.  */
+      /* Avoid wrapping a PTRMEM_CST/REFLECT_EXPR in NOP_EXPR.  */
       conv = copy_node (expr);
       TREE_TYPE (conv) = type;
     }
@@ -871,15 +886,22 @@ ocp_convert (tree type, tree expr, int convtype, int flags,
 	  if (SCOPED_ENUM_P (intype) && (convtype & CONV_STATIC))
 	    e = build_nop (ENUM_UNDERLYING_TYPE (intype), e);
 	  if (complain & tf_warning)
-	    return cp_truthvalue_conversion (e, complain);
+	    e = cp_truthvalue_conversion (e, complain);
 	  else
 	    {
 	      /* Prevent bogus -Wint-in-bool-context warnings coming
 		 from c_common_truthvalue_conversion down the line.  */
 	      warning_sentinel w (warn_int_in_bool_context);
 	      warning_sentinel c (warn_sign_compare);
-	      return cp_truthvalue_conversion (e, complain);
+	      e = cp_truthvalue_conversion (e, complain);
 	    }
+
+	  /* Sometimes boolean types don't match if a non-standard boolean
+	     type has been invented by the target.  */
+	  if (tree e2 = targetm.convert_to_type (type, e))
+	    return e2;
+
+	  return e;
 	}
 
       converted = convert_to_integer_maybe_fold (type, e, dofold);
@@ -973,8 +995,13 @@ ocp_convert (tree type, tree expr, int convtype, int flags,
       if (invalid_nonstatic_memfn_p (loc, expr, complain))
 	/* We displayed the error message.  */;
       else
-	error_at (loc, "conversion from %qH to non-scalar type %qI requested",
-		  TREE_TYPE (expr), type);
+	{
+	  auto_diagnostic_group d;
+	  error_at (loc, "conversion from %qH to non-scalar type %qI requested",
+		    TREE_TYPE (expr), type);
+	  maybe_show_nonconverting_candidate (type, TREE_TYPE (expr), expr,
+					      flags);
+	}
     }
   return error_mark_node;
 }
@@ -1091,6 +1118,7 @@ maybe_warn_nodiscard (tree expr, impl_conv_void implicit)
 		"declared with attribute %<nodiscard%>%s"));
       const char *raw_msg = msg ? (const char *) msg : "";
       auto_diagnostic_group d;
+      auto_urlify_attributes sentinel;
       if (warning_at (loc, OPT_Wunused_result, format, fn, raw_msg))
 	inform (DECL_SOURCE_LOCATION (fn), "declared here");
     }
@@ -1109,6 +1137,7 @@ maybe_warn_nodiscard (tree expr, impl_conv_void implicit)
 		"declared with attribute %<nodiscard%>%s"));
       const char *raw_msg = msg ? (const char *) msg : "";
       auto_diagnostic_group d;
+      auto_urlify_attributes sentinel;
       if (warning_at (loc, OPT_Wunused_result, format, rettype, raw_msg))
 	{
 	  if (fn)
@@ -1123,6 +1152,7 @@ maybe_warn_nodiscard (tree expr, impl_conv_void implicit)
     {
       /* The TARGET_EXPR confuses do_warn_unused_result into thinking that the
 	 result is used, so handle that case here.  */
+      auto_urlify_attributes sentinel;
       if (fn)
 	{
 	  auto_diagnostic_group d;
@@ -1169,13 +1199,6 @@ convert_to_void (tree expr, impl_conv_void implicit, tsubst_flags_t complain)
 
   expr = maybe_undo_parenthesized_ref (expr);
 
-  expr = mark_discarded_use (expr);
-  if (implicit == ICV_CAST)
-    /* An explicit cast to void avoids all -Wunused-but-set* warnings.  */
-    mark_exp_read (expr);
-
-  if (!TREE_TYPE (expr))
-    return expr;
   if (invalid_nonstatic_memfn_p (loc, expr, complain))
     return error_mark_node;
   if (TREE_CODE (expr) == PSEUDO_DTOR_EXPR)
@@ -1190,8 +1213,20 @@ convert_to_void (tree expr, impl_conv_void implicit, tsubst_flags_t complain)
    if (concept_check_p (expr))
      expr = evaluate_concept_check (expr);
 
+  /* Detect using expressions of consteval-only types outside manifestly
+     constant-evaluated contexts.  We are going to discard this expression,
+     so we can't wait till cp_fold_immediate_r.  */
+  if (check_out_of_consteval_use (expr))
+    return error_mark_node;
+
   if (VOID_TYPE_P (TREE_TYPE (expr)))
     return expr;
+
+  expr = mark_discarded_use (expr);
+  if (implicit == ICV_CAST)
+    /* An explicit cast to void avoids all -Wunused-but-set* warnings.  */
+    mark_exp_read (expr);
+
   switch (TREE_CODE (expr))
     {
     case COND_EXPR:
