@@ -1,5 +1,5 @@
 /* Language-independent diagnostic subroutines for the GNU Compiler Collection
-   Copyright (C) 1999-2024 Free Software Foundation, Inc.
+   Copyright (C) 1999-2025 Free Software Foundation, Inc.
    Contributed by Gabriel Dos Reis <gdr@codesourcery.com>
 
 This file is part of GCC.
@@ -23,7 +23,6 @@ along with GCC; see the file COPYING3.  If not see
    message module.  */
 
 #include "config.h"
-#define INCLUDE_MEMORY
 #define INCLUDE_VECTOR
 #include "system.h"
 #include "coretypes.h"
@@ -50,7 +49,6 @@ along with GCC; see the file COPYING3.  If not see
 #include "pretty-print-urlifier.h"
 #include "logical-location.h"
 #include "diagnostic-buffer.h"
-#include "make-unique.h"
 
 #ifdef HAVE_TERMIOS_H
 # include <termios.h>
@@ -97,7 +95,7 @@ int
 get_terminal_width (void)
 {
   const char * s = getenv ("COLUMNS");
-  if (s != NULL) {
+  if (s != nullptr) {
     int n = atoi (s);
     if (n > 0)
       return n;
@@ -224,7 +222,7 @@ diagnostic_context::initialize (int n_opts)
 {
   /* Allocate a basic pretty-printer.  Clients will replace this a
      much more elaborated pretty-printer if they wish.  */
-  m_reference_printer = ::make_unique<pretty_printer> ().release ();
+  m_reference_printer = std::make_unique<pretty_printer> ().release ();
 
   m_file_cache = new file_cache ();
   m_diagnostic_counters.clear ();
@@ -252,10 +250,13 @@ diagnostic_context::initialize (int n_opts)
   m_internal_error = nullptr;
   m_adjust_diagnostic_info = nullptr;
   m_text_callbacks.m_begin_diagnostic = default_diagnostic_text_starter;
-  m_text_callbacks.m_start_span = default_diagnostic_start_span_fn;
+  m_text_callbacks.m_text_start_span
+    = default_diagnostic_start_span_fn<to_text>;
+  m_text_callbacks.m_html_start_span
+    = default_diagnostic_start_span_fn<to_html>;
   m_text_callbacks.m_end_diagnostic = default_diagnostic_text_finalizer;
   m_option_mgr = nullptr;
-  m_urlifier = nullptr;
+  m_urlifier_stack = new auto_vec<urlifier_stack_node> ();
   m_last_location = UNKNOWN_LOCATION;
   m_client_aux_data = nullptr;
   m_lock = 0;
@@ -281,9 +282,12 @@ diagnostic_context::initialize (int n_opts)
   m_tabstop = 8;
   m_escape_format = DIAGNOSTICS_ESCAPE_FORMAT_UNICODE;
   m_edit_context_ptr = nullptr;
-  m_diagnostic_groups.m_nesting_depth = 0;
+  m_diagnostic_groups.m_group_nesting_depth = 0;
+  m_diagnostic_groups.m_diagnostic_nesting_level = 0;
   m_diagnostic_groups.m_emission_count = 0;
-  m_output_sinks.safe_push (new diagnostic_text_output_format (*this, true));
+  m_diagnostic_groups.m_inhibiting_notes_from = 0;
+  m_output_sinks.safe_push
+    (new diagnostic_text_output_format (*this, nullptr, true));
   m_set_locations_cb = nullptr;
   m_client_data_hooks = nullptr;
   m_diagrams.m_theme = nullptr;
@@ -384,7 +388,7 @@ diagnostic_context::finish ()
   /* We might be handling a fatal error.
      Close any active diagnostic groups, which may trigger flushing
      sinks.  */
-  while (m_diagnostic_groups.m_nesting_depth > 0)
+  while (m_diagnostic_groups.m_group_nesting_depth > 0)
     end_group ();
 
   set_diagnostic_buffer (nullptr);
@@ -423,8 +427,13 @@ diagnostic_context::finish ()
   delete m_option_mgr;
   m_option_mgr = nullptr;
 
-  delete m_urlifier;
-  m_urlifier = nullptr;
+  if (m_urlifier_stack)
+    {
+      while (!m_urlifier_stack->is_empty ())
+	pop_urlifier ();
+      delete m_urlifier_stack;
+      m_urlifier_stack = nullptr;
+    }
 
   freeargv (m_original_argv);
   m_original_argv = nullptr;
@@ -439,16 +448,27 @@ diagnostic_context::dump (FILE *out) const
   m_diagnostic_counters.dump (out, 2);
   fprintf (out, "  reference printer:\n");
   m_reference_printer->dump (out, 4);
-  for (unsigned i = 0; i < m_output_sinks.length (); ++i)
+  fprintf (out, "  output sinks:\n");
+  if (m_output_sinks.length () > 0)
     {
-      fprintf (out, "  sink %i:\n", i);
-      m_output_sinks[i]->dump (out, 4);
+      for (unsigned i = 0; i < m_output_sinks.length (); ++i)
+	{
+	  fprintf (out, "  sink %i:\n", i);
+	  m_output_sinks[i]->dump (out, 4);
+	}
     }
+  else
+    fprintf (out, "    (none):\n");
   fprintf (out, "  diagnostic buffer:\n");
   if (m_diagnostic_buffer)
     m_diagnostic_buffer->dump (out, 4);
   else
     fprintf (out, "    (none):\n");
+  fprintf (out, "  file cache:\n");
+  if (m_file_cache)
+    m_file_cache->dump (out, 4);
+  else
+    fprintf (out, "    (none):\n");
 }
 
 /* Return true if sufficiently severe diagnostics have been seen that
@@ -465,11 +485,17 @@ diagnostic_context::execution_failed_p () const
 }
 
 void
-diagnostic_context::
-set_output_format (std::unique_ptr<diagnostic_output_format> output_format)
+diagnostic_context::remove_all_output_sinks ()
 {
   while (!m_output_sinks.is_empty ())
     delete m_output_sinks.pop ();
+}
+
+void
+diagnostic_context::
+set_output_format (std::unique_ptr<diagnostic_output_format> output_format)
+{
+  remove_all_output_sinks ();
   m_output_sinks.safe_push (output_format.release ());
 }
 
@@ -498,6 +524,13 @@ diagnostic_context::supports_fnotice_on_stderr_p () const
   return true;
 }
 
+void
+diagnostic_context::set_main_input_filename (const char *filename)
+{
+  for (auto sink : m_output_sinks)
+    sink->set_main_input_filename (filename);
+}
+
 void
 diagnostic_context::
 set_client_data_hooks (std::unique_ptr<diagnostic_client_data_hooks> hooks)
@@ -531,13 +564,51 @@ set_option_manager (std::unique_ptr<diagnostic_option_manager> mgr,
 }
 
 void
-diagnostic_context::set_urlifier (std::unique_ptr<urlifier> urlifier)
+diagnostic_context::push_owned_urlifier (std::unique_ptr<urlifier> ptr)
 {
-  delete m_urlifier;
-  /* Ideally the field would be a std::unique_ptr here.  */
-  m_urlifier = urlifier.release ();
+  gcc_assert (m_urlifier_stack);
+  const urlifier_stack_node node = { ptr.release (), true };
+  m_urlifier_stack->safe_push (node);
 }
 
+void
+diagnostic_context::push_borrowed_urlifier (const urlifier &loan)
+{
+  gcc_assert (m_urlifier_stack);
+  const urlifier_stack_node node = { const_cast <urlifier *> (&loan), false };
+  m_urlifier_stack->safe_push (node);
+}
+
+void
+diagnostic_context::pop_urlifier ()
+{
+  gcc_assert (m_urlifier_stack);
+  gcc_assert (m_urlifier_stack->length () > 0);
+
+  const urlifier_stack_node node = m_urlifier_stack->pop ();
+  if (node.m_owned)
+    delete node.m_urlifier;
+}
+
+const logical_location_manager *
+diagnostic_context::get_logical_location_manager () const
+{
+  if (!m_client_data_hooks)
+    return nullptr;
+  return m_client_data_hooks->get_logical_location_manager ();
+}
+
+const urlifier *
+diagnostic_context::get_urlifier () const
+{
+  if (!m_urlifier_stack)
+    return nullptr;
+  if (m_urlifier_stack->is_empty ())
+    return nullptr;
+  return m_urlifier_stack->last ().m_urlifier;
+}
+
+
 /* Set PP as the reference printer for this context.
    Refresh all output sinks.  */
 
@@ -608,7 +679,7 @@ diagnostic_set_info_translated (diagnostic_info *diagnostic, const char *msg,
   diagnostic->message.m_format_spec = msg;
   diagnostic->message.m_richloc = richloc;
   diagnostic->richloc = richloc;
-  diagnostic->metadata = NULL;
+  diagnostic->metadata = nullptr;
   diagnostic->kind = kind;
   diagnostic->option_id = 0;
 }
@@ -628,11 +699,11 @@ static const char *const diagnostic_kind_color[] = {
 #define DEFINE_DIAGNOSTIC_KIND(K, T, C) (C),
 #include "diagnostic.def"
 #undef DEFINE_DIAGNOSTIC_KIND
-  NULL
+  nullptr
 };
 
 /* Get a color name for diagnostics of type KIND
-   Result could be NULL.  */
+   Result could be nullptr.  */
 
 const char *
 diagnostic_get_color_for_kind (diagnostic_t kind)
@@ -766,12 +837,12 @@ bt_callback (void *data, uintptr_t pc, const char *filename, int lineno,
 
   /* If we don't have any useful information, don't print
      anything.  */
-  if (filename == NULL && function == NULL)
+  if (filename == nullptr && function == nullptr)
     return 0;
 
   /* Skip functions in diagnostic.cc.  */
   if (*pcount == 0
-      && filename != NULL
+      && filename != nullptr
       && strcmp (lbasename (filename), "diagnostic.cc") == 0)
     return 0;
 
@@ -784,13 +855,13 @@ bt_callback (void *data, uintptr_t pc, const char *filename, int lineno,
     }
   ++*pcount;
 
-  char *alc = NULL;
-  if (function != NULL)
+  char *alc = nullptr;
+  if (function != nullptr)
     {
       char *str = cplus_demangle_v3 (function,
 				     (DMGL_VERBOSE | DMGL_ANSI
 				      | DMGL_GNU_V3 | DMGL_PARAMS));
-      if (str != NULL)
+      if (str != nullptr)
 	{
 	  alc = str;
 	  function = str;
@@ -802,7 +873,7 @@ bt_callback (void *data, uintptr_t pc, const char *filename, int lineno,
 	  if (strncmp (function, bt_stop[i], len) == 0
 	      && (function[len] == '\0' || function[len] == '('))
 	    {
-	      if (alc != NULL)
+	      if (alc != nullptr)
 		free (alc);
 	      /* Returning a non-zero value stops the backtrace.  */
 	      return 1;
@@ -812,11 +883,11 @@ bt_callback (void *data, uintptr_t pc, const char *filename, int lineno,
 
   fprintf (stderr, "0x%lx %s\n\t%s:%d\n",
 	   (unsigned long) pc,
-	   function == NULL ? "???" : function,
-	   filename == NULL ? "???" : filename,
+	   function == nullptr ? "???" : function,
+	   filename == nullptr ? "???" : filename,
 	   lineno);
 
-  if (alc != NULL)
+  if (alc != nullptr)
     free (alc);
 
   return 0;
@@ -865,6 +936,7 @@ diagnostic_context::check_max_errors (bool flush)
 
 /* Take any action which is expected to happen after the diagnostic
    is written out.  This function does not always return.  */
+
 void
 diagnostic_context::action_after_output (diagnostic_t diag_kind)
 {
@@ -901,11 +973,11 @@ diagnostic_context::action_after_output (diagnostic_t diag_kind)
 	    finish ();
 	  }
 
-	struct backtrace_state *state = NULL;
+	struct backtrace_state *state = nullptr;
 	if (diag_kind == DK_ICE)
-	  state = backtrace_create_state (NULL, 0, bt_err_callback, NULL);
+	  state = backtrace_create_state (nullptr, 0, bt_err_callback, nullptr);
 	int count = 0;
-	if (state != NULL)
+	if (state != nullptr)
 	  backtrace_full (state, 2, bt_callback, bt_err_callback,
 			  (void *) &count);
 
@@ -939,47 +1011,76 @@ diagnostic_context::action_after_output (diagnostic_t diag_kind)
     }
 }
 
-/* class logical_location.  */
+/* State whether we should inhibit notes in the current diagnostic_group and
+   its future children if any.  */
+
+void
+diagnostic_context::inhibit_notes_in_group (bool inhibit)
+{
+  int curr_depth = (m_diagnostic_groups.m_group_nesting_depth
+		    + m_diagnostic_groups.m_diagnostic_nesting_level);
+
+  if (inhibit)
+    {
+      /* If we're already inhibiting, there's nothing to do.  */
+      if (m_diagnostic_groups.m_inhibiting_notes_from)
+	return;
+
+      /* Since we're called via warning/error/... that all have their own
+	 diagnostic_group, we must consider that we started inhibiting in their
+	 parent.  */
+      gcc_assert (m_diagnostic_groups.m_group_nesting_depth > 0);
+      m_diagnostic_groups.m_inhibiting_notes_from = curr_depth - 1;
+    }
+  else if (m_diagnostic_groups.m_inhibiting_notes_from)
+    {
+      /* Only cancel inhibition at the depth that set it up.  */
+      if (curr_depth >= m_diagnostic_groups.m_inhibiting_notes_from)
+	return;
+
+      m_diagnostic_groups.m_inhibiting_notes_from = 0;
+    }
+}
+
+/* Return whether notes must be inhibited in the current diagnostic_group.  */
+
+bool
+diagnostic_context::notes_inhibited_in_group () const
+{
+  if (m_diagnostic_groups.m_inhibiting_notes_from
+      && (m_diagnostic_groups.m_group_nesting_depth
+	  + m_diagnostic_groups.m_diagnostic_nesting_level
+	  >= m_diagnostic_groups.m_inhibiting_notes_from))
+    return true;
+  return false;
+}
+
+/* class logical_location_manager.  */
 
 /* Return true iff this is a function or method.  */
 
 bool
-logical_location::function_p () const
+logical_location_manager::function_p (key k) const
 {
-  switch (get_kind ())
+  switch (get_kind (k))
     {
     default:
       gcc_unreachable ();
-    case LOGICAL_LOCATION_KIND_UNKNOWN:
-    case LOGICAL_LOCATION_KIND_MODULE:
-    case LOGICAL_LOCATION_KIND_NAMESPACE:
-    case LOGICAL_LOCATION_KIND_TYPE:
-    case LOGICAL_LOCATION_KIND_RETURN_TYPE:
-    case LOGICAL_LOCATION_KIND_PARAMETER:
-    case LOGICAL_LOCATION_KIND_VARIABLE:
+    case logical_location_kind::unknown:
+    case logical_location_kind::module_:
+    case logical_location_kind::namespace_:
+    case logical_location_kind::type:
+    case logical_location_kind::return_type:
+    case logical_location_kind::parameter:
+    case logical_location_kind::variable:
       return false;
 
-    case LOGICAL_LOCATION_KIND_FUNCTION:
-    case LOGICAL_LOCATION_KIND_MEMBER:
+    case logical_location_kind::function:
+    case logical_location_kind::member:
       return true;
     }
 }
 
-void
-default_diagnostic_start_span_fn (const diagnostic_location_print_policy &loc_policy,
-				  pretty_printer *pp,
-				  expanded_location exploc)
-{
-  const diagnostic_column_policy &column_policy
-    = loc_policy.get_column_policy ();
-  label_text text
-    = column_policy.get_location_text (exploc,
-				       loc_policy.show_column_p (),
-				       pp_show_color (pp));
-  pp_string (pp, text.get ());
-  pp_newline (pp);
-}
-
 /* Interface to specify diagnostic kind overrides.  Returns the
    previous setting, or DK_UNSPECIFIED if the parameters are out of
    range.  If OPTION_ID is zero, the new setting is for all the
@@ -1093,7 +1194,7 @@ print_parseable_fixits (file_cache &fc,
   gcc_assert (richloc);
 
   char *saved_prefix = pp_take_prefix (pp);
-  pp_set_prefix (pp, NULL);
+  pp_set_prefix (pp, nullptr);
 
   for (unsigned i = 0; i < richloc->get_num_fixit_hints (); i++)
     {
@@ -1322,14 +1423,17 @@ diagnostic_context::report_diagnostic (diagnostic_info *diagnostic)
   /* Every call to report_diagnostic should be within a
      begin_group/end_group pair so that output formats can reliably
      flush diagnostics with on_end_group when the topmost group is ended.  */
-  gcc_assert (m_diagnostic_groups.m_nesting_depth > 0);
+  gcc_assert (m_diagnostic_groups.m_group_nesting_depth > 0);
 
   /* Give preference to being able to inhibit warnings, before they
      get reclassified to something else.  */
   bool was_warning = (diagnostic->kind == DK_WARNING
 		      || diagnostic->kind == DK_PEDWARN);
   if (was_warning && m_inhibit_warnings)
-    return false;
+    {
+      inhibit_notes_in_group ();
+      return false;
+    }
 
   if (m_adjust_diagnostic_info)
     m_adjust_diagnostic_info (this, diagnostic);
@@ -1345,18 +1449,6 @@ diagnostic_context::report_diagnostic (diagnostic_info *diagnostic)
   if (diagnostic->kind == DK_NOTE && m_inhibit_notes_p)
     return false;
 
-  if (m_lock > 0)
-    {
-      /* If we're reporting an ICE in the middle of some other error,
-	 try to flush out the previous error, then let this one
-	 through.  Don't do this more than once.  */
-      if ((diagnostic->kind == DK_ICE || diagnostic->kind == DK_ICE_NOBT)
-	  && m_lock == 1)
-	pp_newline_and_flush (m_reference_printer);
-      else
-	error_recursion ();
-    }
-
   /* If the user requested that warnings be treated as errors, so be
      it.  Note that we do this before the next block so that
      individual warnings can be overridden back to warnings with
@@ -1371,7 +1463,10 @@ diagnostic_context::report_diagnostic (diagnostic_info *diagnostic)
      not disabled by #pragma GCC diagnostic anywhere along the inlining
      stack.  .  */
   if (!diagnostic_enabled (diagnostic))
-    return false;
+    {
+      inhibit_notes_in_group ();
+      return false;
+    }
 
   if ((was_warning || diagnostic->kind == DK_WARNING)
       && ((!m_warn_system_headers
@@ -1381,9 +1476,28 @@ diagnostic_context::report_diagnostic (diagnostic_info *diagnostic)
        inlining stack (if there is one) are in system headers.  */
     return false;
 
+  if (diagnostic->kind == DK_NOTE && notes_inhibited_in_group ())
+    /* Bail for all the notes in the diagnostic_group that started to inhibit notes.  */
+    return false;
+
   if (diagnostic->kind != DK_NOTE && diagnostic->kind != DK_ICE)
     check_max_errors (false);
 
+  if (m_lock > 0)
+    {
+      /* If we're reporting an ICE in the middle of some other error,
+	 try to flush out the previous error, then let this one
+	 through.  Don't do this more than once.  */
+      if ((diagnostic->kind == DK_ICE || diagnostic->kind == DK_ICE_NOBT)
+	  && m_lock == 1)
+	pp_newline_and_flush (m_reference_printer);
+      else
+	error_recursion ();
+    }
+
+  /* We are accepting the diagnostic, so should stop inhibiting notes.  */
+  inhibit_notes_in_group (/*inhibit=*/false);
+
   m_lock++;
 
   if (diagnostic->kind == DK_ICE || diagnostic->kind == DK_ICE_NOBT)
@@ -1480,7 +1594,7 @@ diagnostic_context::report_diagnostic (diagnostic_info *diagnostic)
       || diagnostic->kind == DK_ICE
       || diagnostic->kind == DK_ICE_NOBT)
     action_after_output (diagnostic->kind);
-  diagnostic->x_data = NULL;
+  diagnostic->x_data = nullptr;
 
   if (m_edit_context_ptr)
     if (diagnostic->richloc->fixits_can_be_auto_applied_p ())
@@ -1510,6 +1624,14 @@ diagnostic_context::report_verbatim (text_info &text)
     }
 }
 
+void
+diagnostic_context::
+report_global_digraph (const diagnostics::digraphs::lazy_digraph &ldg)
+{
+  for (auto sink : m_output_sinks)
+    sink->report_global_digraph (ldg);
+}
+
 /* Get the number of digits in the decimal representation of VALUE.  */
 
 int
@@ -1679,9 +1801,9 @@ fancy_abort (const char *file, int line, const char *function)
 
       /* Attempt to print a backtrace.  */
       struct backtrace_state *state
-	= backtrace_create_state (NULL, 0, bt_err_callback, NULL);
+	= backtrace_create_state (nullptr, 0, bt_err_callback, nullptr);
       int count = 0;
-      if (state != NULL)
+      if (state != nullptr)
 	backtrace_full (state, 2, bt_callback, bt_err_callback,
 			(void *) &count);
 
@@ -1701,13 +1823,13 @@ fancy_abort (const char *file, int line, const char *function)
 void
 diagnostic_context::begin_group ()
 {
-  m_diagnostic_groups.m_nesting_depth++;
+  m_diagnostic_groups.m_group_nesting_depth++;
 }
 
 void
 diagnostic_context::end_group ()
 {
-  if (--m_diagnostic_groups.m_nesting_depth == 0)
+  if (--m_diagnostic_groups.m_group_nesting_depth == 0)
     {
       /* Handle the case where we've popped the final diagnostic group.
 	 If any diagnostics were emitted, give the context a chance
@@ -1717,6 +1839,22 @@ diagnostic_context::end_group ()
 	  sink->on_end_group ();
       m_diagnostic_groups.m_emission_count = 0;
     }
+  /* We're popping one level, so might need to stop inhibiting notes.  */
+  inhibit_notes_in_group (/*inhibit=*/false);
+}
+
+void
+diagnostic_context::push_nesting_level ()
+{
+  ++m_diagnostic_groups.m_diagnostic_nesting_level;
+}
+
+void
+diagnostic_context::pop_nesting_level ()
+{
+  --m_diagnostic_groups.m_diagnostic_nesting_level;
+  /* We're popping one level, so might need to stop inhibiting notes.  */
+  inhibit_notes_in_group (/*inhibit=*/false);
 }
 
 void
@@ -1742,6 +1880,7 @@ diagnostic_output_format_init (diagnostic_context &context,
 			       enum diagnostics_output_format format,
 			       bool json_formatting)
 {
+  diagnostic_output_format *new_sink = nullptr;
   switch (format)
     {
     default:
@@ -1750,39 +1889,26 @@ diagnostic_output_format_init (diagnostic_context &context,
       /* The default; do nothing.  */
       break;
 
-    case DIAGNOSTICS_OUTPUT_FORMAT_JSON_STDERR:
-      diagnostic_output_format_init_json_stderr (context,
-						 json_formatting);
-      break;
-
-    case DIAGNOSTICS_OUTPUT_FORMAT_JSON_FILE:
-      diagnostic_output_format_init_json_file (context,
-					       json_formatting,
-					       base_file_name);
-      break;
-
     case DIAGNOSTICS_OUTPUT_FORMAT_SARIF_STDERR:
-      diagnostic_output_format_init_sarif_stderr (context,
-						  line_table,
-						  main_input_filename_,
-						  json_formatting,
-						  sarif_version::v2_1_0);
+      new_sink = &diagnostic_output_format_init_sarif_stderr (context,
+							      line_table,
+							      json_formatting);
       break;
 
     case DIAGNOSTICS_OUTPUT_FORMAT_SARIF_FILE:
-      diagnostic_output_format_init_sarif_file (context,
-						line_table,
-						main_input_filename_,
-						json_formatting,
-						sarif_version::v2_1_0,
-						base_file_name);
+      new_sink = &diagnostic_output_format_init_sarif_file (context,
+							line_table,
+							    json_formatting,
+							    base_file_name);
       break;
     }
+  if (new_sink)
+    new_sink->set_main_input_filename (main_input_filename_);
 }
 
 /* Initialize this context's m_diagrams based on CHARSET.
    Specifically, make a text_art::theme object for m_diagrams.m_theme,
-   (or NULL for "no diagrams").  */
+   (or nullptr for "no diagrams").  */
 
 void
 diagnostic_context::
@@ -1824,7 +1950,11 @@ diagnostic_context::set_diagnostic_buffer (diagnostic_buffer *buffer)
   /* We don't allow changing buffering within a diagnostic group
      (to simplify handling of buffered diagnostics within the
      diagnostic_format implementations).  */
-  gcc_assert (m_diagnostic_groups.m_nesting_depth == 0);
+  gcc_assert (m_diagnostic_groups.m_group_nesting_depth == 0);
+
+  /* Likewise, for simplicity, we only allow changing buffers
+     at nesting level 0.  */
+  gcc_assert (m_diagnostic_groups.m_diagnostic_nesting_level == 0);
 
   m_diagnostic_buffer = buffer;
 
@@ -2103,7 +2233,7 @@ test_print_parseable_fixits_insert ()
 
   linemap_add (line_table, LC_ENTER, false, "test.c", 0);
   linemap_line_start (line_table, 5, 100);
-  linemap_add (line_table, LC_LEAVE, false, NULL, 0);
+  linemap_add (line_table, LC_LEAVE, false, nullptr, 0);
   location_t where = linemap_position_for_column (line_table, 10);
   richloc.add_fixit_insert_before (where, "added content");
 
@@ -2124,7 +2254,7 @@ test_print_parseable_fixits_remove ()
 
   linemap_add (line_table, LC_ENTER, false, "test.c", 0);
   linemap_line_start (line_table, 5, 100);
-  linemap_add (line_table, LC_LEAVE, false, NULL, 0);
+  linemap_add (line_table, LC_LEAVE, false, nullptr, 0);
   source_range where;
   where.m_start = linemap_position_for_column (line_table, 10);
   where.m_finish = linemap_position_for_column (line_table, 20);
@@ -2147,7 +2277,7 @@ test_print_parseable_fixits_replace ()
 
   linemap_add (line_table, LC_ENTER, false, "test.c", 0);
   linemap_line_start (line_table, 5, 100);
-  linemap_add (line_table, LC_LEAVE, false, NULL, 0);
+  linemap_add (line_table, LC_LEAVE, false, nullptr, 0);
   source_range where;
   where.m_start = linemap_position_for_column (line_table, 10);
   where.m_finish = linemap_position_for_column (line_table, 20);
@@ -2178,7 +2308,7 @@ test_print_parseable_fixits_bytes_vs_display_columns ()
 
   linemap_add (line_table, LC_ENTER, false, fname, 0);
   linemap_line_start (line_table, 1, 100);
-  linemap_add (line_table, LC_LEAVE, false, NULL, 0);
+  linemap_add (line_table, LC_LEAVE, false, nullptr, 0);
   source_range where;
   where.m_start = linemap_position_for_column (line_table, 12);
   where.m_finish = linemap_position_for_column (line_table, 17);
@@ -2236,7 +2366,7 @@ assert_location_text (const char *expected_loc_text,
   xloc.file = filename;
   xloc.line = line;
   xloc.column = column;
-  xloc.data = NULL;
+  xloc.data = nullptr;
   xloc.sysp = false;
 
   diagnostic_column_policy column_policy (dc);
@@ -2252,7 +2382,7 @@ test_get_location_text ()
 {
   const char *old_progname = progname;
   progname = "PROGNAME";
-  assert_location_text ("PROGNAME:", NULL, 0, 0, true);
+  assert_location_text ("PROGNAME:", nullptr, 0, 0, true);
   char *built_in_colon = concat (special_fname_builtin (), ":", (char *) 0);
   assert_location_text (built_in_colon, special_fname_builtin (),
 			42, 10, true);
