@@ -1,5 +1,5 @@
 /* Various declarations for language-independent diagnostics subroutines.
-   Copyright (C) 2000-2024 Free Software Foundation, Inc.
+   Copyright (C) 2000-2025 Free Software Foundation, Inc.
    Contributed by Gabriel Dos Reis <gdr@codesourcery.com>
 
 This file is part of GCC.
@@ -21,14 +21,6 @@ along with GCC; see the file COPYING3.  If not see
 #ifndef GCC_DIAGNOSTIC_H
 #define GCC_DIAGNOSTIC_H
 
-/* This header uses std::unique_ptr, but <memory> can't be directly
-   included due to issues with macros.  Hence it must be included from
-   system.h by defining INCLUDE_MEMORY in any source file using it.  */
-
-#ifndef INCLUDE_MEMORY
-# error "You must define INCLUDE_MEMORY before including system.h to use diagnostic.h"
-#endif
-
 #include "unique-argv.h"
 #include "rich-location.h"
 #include "pretty-print.h"
@@ -39,6 +31,11 @@ namespace text_art
   class theme;
 } // namespace text_art
 
+namespace xml
+{
+  class printer;
+} // namespace xml
+
 /* An enum for controlling what units to use for the column number
    when diagnostics are output, used by the -fdiagnostics-column-unit option.
    Tabs will be expanded or not according to the value of -ftabstop.  The origin
@@ -80,10 +77,10 @@ enum diagnostics_output_format
   /* JSON-based output, to a file.  */
   DIAGNOSTICS_OUTPUT_FORMAT_JSON_FILE,
 
-  /* SARIF-based output, to stderr.  */
+  /* SARIF-based output, as JSON to stderr.  */
   DIAGNOSTICS_OUTPUT_FORMAT_SARIF_STDERR,
 
-  /* SARIF-based output, to a file.  */
+  /* SARIF-based output, to a JSON file.  */
   DIAGNOSTICS_OUTPUT_FORMAT_SARIF_FILE
 };
 
@@ -185,10 +182,15 @@ class diagnostic_source_print_policy;
 typedef void (*diagnostic_text_starter_fn) (diagnostic_text_output_format &,
 					    const diagnostic_info *);
 
-typedef void
-(*diagnostic_start_span_fn) (const diagnostic_location_print_policy &,
-			     pretty_printer *,
-			     expanded_location);
+struct to_text;
+struct to_html;
+
+extern pretty_printer *get_printer (to_text &);
+
+template <typename Sink>
+using diagnostic_start_span_fn = void (*) (const diagnostic_location_print_policy &,
+					   Sink &sink,
+					   expanded_location);
 
 typedef void (*diagnostic_text_finalizer_fn) (diagnostic_text_output_format &,
 					      const diagnostic_info *,
@@ -225,7 +227,7 @@ public:
 
 class edit_context;
 class diagnostic_client_data_hooks;
-class logical_location;
+class logical_location_manager;
 class diagnostic_diagram;
 class diagnostic_source_effect_info;
 class diagnostic_output_format;
@@ -397,11 +399,32 @@ public:
   const diagnostic_column_policy &
   get_column_policy () const { return m_column_policy; }
 
+  void
+  print_text_span_start (const diagnostic_context &dc,
+			 pretty_printer &pp,
+			 const expanded_location &exploc);
+
+  void
+  print_html_span_start (const diagnostic_context &dc,
+			 xml::printer &xp,
+			 const expanded_location &exploc);
+
 private:
   diagnostic_column_policy m_column_policy;
   bool m_show_column;
 };
 
+/* Abstract base class for optionally supplying extra tags when writing
+   out annotation labels in HTML output.  */
+
+class html_label_writer
+{
+public:
+  virtual ~html_label_writer () {}
+  virtual void begin_label () = 0;
+  virtual void end_label () = 0;
+};
+
 /* A bundle of state for printing source within a diagnostic,
    to isolate the interactions between diagnostic_context and the
    implementation of diagnostic_show_locus.  */
@@ -410,6 +433,8 @@ class diagnostic_source_print_policy
 {
 public:
   diagnostic_source_print_policy (const diagnostic_context &);
+  diagnostic_source_print_policy (const diagnostic_context &,
+				  const diagnostic_source_printing_options &);
 
   void
   print (pretty_printer &pp,
@@ -417,11 +442,21 @@ public:
 	 diagnostic_t diagnostic_kind,
 	 diagnostic_source_effect_info *effect_info) const;
 
+  void
+  print_as_html (xml::printer &xp,
+		 const rich_location &richloc,
+		 diagnostic_t diagnostic_kind,
+		 diagnostic_source_effect_info *effect_info,
+		 html_label_writer *label_writer) const;
+
   const diagnostic_source_printing_options &
   get_options () const { return m_options; }
 
-  diagnostic_start_span_fn
-  get_start_span_fn () const { return m_start_span_cb; }
+  diagnostic_start_span_fn<to_text>
+  get_text_start_span_fn () const { return m_text_start_span_cb; }
+
+  diagnostic_start_span_fn<to_html>
+  get_html_start_span_fn () const { return m_html_start_span_cb; }
 
   file_cache &
   get_file_cache () const { return m_file_cache; }
@@ -448,7 +483,8 @@ public:
 private:
   const diagnostic_source_printing_options &m_options;
   class diagnostic_location_print_policy m_location_policy;
-  diagnostic_start_span_fn m_start_span_cb;
+  diagnostic_start_span_fn<to_text> m_text_start_span_cb;
+  diagnostic_start_span_fn<to_html> m_html_start_span_cb;
   file_cache &m_file_cache;
 
   /* Other data copied from diagnostic_context.  */
@@ -475,15 +511,46 @@ struct diagnostic_counters
   int m_count_for_kind[DK_LAST_DIAGNOSTIC_KIND];
 };
 
-/* This data structure bundles altogether any information relevant to
-   the context of a diagnostic message.  */
+/* This class encapsulates the state of the diagnostics subsystem
+   as a whole (either directly, or via owned objects of other classes, to
+   avoid global variables).
+
+   It has responsibility for:
+   - being a central place for clients to report diagnostics
+   - reporting those diagnostics to zero or more output sinks
+     (e.g. text vs SARIF)
+   - providing a "dump" member function for a debug dump of the state of
+     the diagnostics subsytem
+   - direct vs buffered diagnostics (see class diagnostic_buffer)
+   - tracking the original argv of the program (for SARIF output)
+   - crash-handling
+
+   It delegates responsibility to various other classes:
+   - the various output sinks (instances of diagnostic_output_format
+     subclasses)
+   - formatting of messages (class pretty_printer)
+   - an optional urlifier to inject URLs into formatted messages
+   - counting the number of diagnostics reported of each kind
+     (class diagnostic_counters)
+   - calling out to a diagnostic_option_manager to determine if
+     a particular warning is enabled or disabled
+   - tracking pragmas that enable/disable warnings in a range of
+     source code
+   - a cache for use when quoting the user's source code (class file_cache)
+   - a text_art::theme
+   - an edit_context for generating patches from fix-it hints
+   - diagnostic_client_data_hooks for metadata.
+
+   Try to avoid adding new responsibilities to this class itself, to avoid
+   the "blob" anti-pattern.  */
+
 class diagnostic_context
 {
 public:
   /* Give access to m_text_callbacks.  */
   friend diagnostic_text_starter_fn &
   diagnostic_text_starter (diagnostic_context *context);
-  friend diagnostic_start_span_fn &
+  friend diagnostic_start_span_fn<to_text> &
   diagnostic_start_span (diagnostic_context *context);
   friend diagnostic_text_finalizer_fn &
   diagnostic_text_finalizer (diagnostic_context *context);
@@ -526,6 +593,9 @@ public:
   void begin_group ();
   void end_group ();
 
+  void push_nesting_level ();
+  void pop_nesting_level ();
+
   bool warning_enabled_at (location_t loc, diagnostic_option_id option_id);
 
   bool option_unspecified_p (diagnostic_option_id option_id) const
@@ -570,9 +640,16 @@ public:
   }
 
   void maybe_show_locus (const rich_location &richloc,
+			 const diagnostic_source_printing_options &opts,
 			 diagnostic_t diagnostic_kind,
 			 pretty_printer &pp,
 			 diagnostic_source_effect_info *effect_info);
+  void maybe_show_locus_as_html (const rich_location &richloc,
+				 const diagnostic_source_printing_options &opts,
+				 diagnostic_t diagnostic_kind,
+				 xml::printer &xp,
+				 diagnostic_source_effect_info *effect_info,
+				 html_label_writer *label_writer);
 
   void emit_diagram (const diagnostic_diagram &diagram);
 
@@ -580,7 +657,11 @@ public:
   void set_output_format (std::unique_ptr<diagnostic_output_format> output_format);
   void set_text_art_charset (enum diagnostic_text_art_charset charset);
   void set_client_data_hooks (std::unique_ptr<diagnostic_client_data_hooks> hooks);
-  void set_urlifier (std::unique_ptr<urlifier>);
+
+  void push_owned_urlifier (std::unique_ptr<urlifier>);
+  void push_borrowed_urlifier (const urlifier &);
+  void pop_urlifier ();
+
   void create_edit_context ();
   void set_warning_as_error_requested (bool val)
   {
@@ -637,7 +718,12 @@ public:
   {
     return m_client_data_hooks;
   }
-  urlifier *get_urlifier () const { return m_urlifier; }
+
+  const logical_location_manager *
+  get_logical_location_manager () const;
+
+  const urlifier *get_urlifier () const;
+
   text_art::theme *get_diagram_theme () const { return m_diagrams.m_theme; }
 
   int &diagnostic_count (diagnostic_t kind)
@@ -692,6 +778,13 @@ public:
 			  const char *, const char *, va_list *,
 			  diagnostic_t) ATTRIBUTE_GCC_DIAG(7,0);
 
+  int get_diagnostic_nesting_level () const
+  {
+    return m_diagnostic_groups.m_diagnostic_nesting_level;
+  }
+
+  char *build_indent_prefix () const;
+
   int
   pch_save (FILE *f)
   {
@@ -726,8 +819,17 @@ public:
   void
   add_sink (std::unique_ptr<diagnostic_output_format>);
 
+  void remove_all_output_sinks ();
+
   bool supports_fnotice_on_stderr_p () const;
 
+  /* Raise SIGABRT on any diagnostic of severity DK_ERROR or higher.  */
+  void
+  set_abort_on_error (bool val)
+  {
+    m_abort_on_error = val;
+  }
+
 private:
   void error_recursion () ATTRIBUTE_NORETURN;
 
@@ -784,10 +886,10 @@ private:
      each diagnostic, if known.  */
   bool m_show_option_requested;
 
-public:
   /* True if we should raise a SIGABRT on errors.  */
   bool m_abort_on_error;
 
+public:
   /* True if we should show the column number on diagnostics.  */
   bool m_show_column;
 
@@ -797,9 +899,9 @@ public:
   /* True if permerrors are warnings.  */
   bool m_permissive;
 
-  /* The index of the option to associate with turning permerrors into
-     warnings.  */
-  int m_opt_permissive;
+  /* The option to associate with turning permerrors into warnings,
+     if any.  */
+  diagnostic_option_id m_opt_permissive;
 
   /* True if errors are fatal.  */
   bool m_fatal_errors;
@@ -828,7 +930,8 @@ private:
     /* This function is called by diagnostic_show_locus in between
        disjoint spans of source code, so that the context can print
        something to indicate that a new span of source code has begun.  */
-    diagnostic_start_span_fn m_start_span;
+    diagnostic_start_span_fn<to_text> m_text_start_span;
+    diagnostic_start_span_fn<to_html> m_html_start_span;
 
     /* This function is called after the diagnostic message is printed.  */
     diagnostic_text_finalizer_fn m_end_diagnostic;
@@ -848,11 +951,16 @@ private:
   diagnostic_option_manager *m_option_mgr;
   unsigned m_lang_mask;
 
-  /* An optional hook for adding URLs to quoted text strings in
+  /* A stack of optional hooks for adding URLs to quoted text strings in
      diagnostics.  Only used for the main diagnostic message.
-     Owned by the context; this would be a std::unique_ptr if
-     diagnostic_context had a proper ctor.  */
-  urlifier *m_urlifier;
+     Typically a single one owner by the context, but can be temporarily
+     overridden by a borrowed urlifier (e.g. on-stack).  */
+  struct urlifier_stack_node
+  {
+    urlifier *m_urlifier;
+    bool m_owned;
+  };
+  auto_vec<urlifier_stack_node> *m_urlifier_stack;
 
 public:
   /* Auxiliary data for client.  */
@@ -902,13 +1010,22 @@ private:
   /* Fields relating to diagnostic groups.  */
   struct {
     /* How many diagnostic_group instances are currently alive.  */
-    int m_nesting_depth;
+    int m_group_nesting_depth;
+
+    /* How many nesting levels have been pushed within this group.  */
+    int m_diagnostic_nesting_level;
 
     /* How many diagnostics have been emitted since the bottommost
        diagnostic_group was pushed.  */
     int m_emission_count;
+
+    /* The "group+diagnostic" nesting depth from which to inhibit notes.  */
+    int m_inhibiting_notes_from;
   } m_diagnostic_groups;
 
+  void inhibit_notes_in_group (bool inhibit = true);
+  bool notes_inhibited_in_group () const;
+
   /* The various sinks to which diagnostics are to be outputted
      (text vs structured formats such as SARIF).
      The sinks are owned by the context; this would be a
@@ -972,10 +1089,10 @@ diagnostic_text_starter (diagnostic_context *context)
 /* Client supplied function called between disjoint spans of source code,
    so that the context can print
    something to indicate that a new span of source code has begun.  */
-inline diagnostic_start_span_fn &
+inline diagnostic_start_span_fn<to_text> &
 diagnostic_start_span (diagnostic_context *context)
 {
-  return context->m_text_callbacks.m_start_span;
+  return context->m_text_callbacks.m_text_start_span;
 }
 
 /* Client supplied function called after a diagnostic message is
@@ -990,13 +1107,6 @@ diagnostic_text_finalizer (diagnostic_context *context)
 #define diagnostic_context_auxiliary_data(DC) (DC)->m_client_aux_data
 #define diagnostic_info_auxiliary_data(DI) (DI)->x_data
 
-/* Raise SIGABRT on any diagnostic of severity DK_ERROR or higher.  */
-inline void
-diagnostic_abort_on_error (diagnostic_context *context)
-{
-  context->m_abort_on_error = true;
-}
-
 /* This diagnostic_context is used by front-ends that directly output
    diagnostic messages without going through `error', `warning',
    and similar functions.  */
@@ -1055,6 +1165,7 @@ diagnostic_finish (diagnostic_context *context)
 
 inline void
 diagnostic_show_locus (diagnostic_context *context,
+		       const diagnostic_source_printing_options &opts,
 		       rich_location *richloc,
 		       diagnostic_t diagnostic_kind,
 		       pretty_printer *pp,
@@ -1063,7 +1174,22 @@ diagnostic_show_locus (diagnostic_context *context,
   gcc_assert (context);
   gcc_assert (richloc);
   gcc_assert (pp);
-  context->maybe_show_locus (*richloc, diagnostic_kind, *pp, effect_info);
+  context->maybe_show_locus (*richloc, opts, diagnostic_kind, *pp, effect_info);
+}
+
+inline void
+diagnostic_show_locus_as_html (diagnostic_context *context,
+			       const diagnostic_source_printing_options &opts,
+			       rich_location *richloc,
+			       diagnostic_t diagnostic_kind,
+			       xml::printer &xp,
+			       diagnostic_source_effect_info *effect_info = nullptr,
+			       html_label_writer *label_writer = nullptr)
+{
+  gcc_assert (context);
+  gcc_assert (richloc);
+  context->maybe_show_locus_as_html (*richloc, opts, diagnostic_kind, xp,
+				     effect_info, label_writer);
 }
 
 /* Because we read source files a second time after the frontend did it the
@@ -1139,8 +1265,9 @@ extern void diagnostic_set_info_translated (diagnostic_info *, const char *,
 #endif
 void default_diagnostic_text_starter (diagnostic_text_output_format &,
 				      const diagnostic_info *);
+template <typename Sink>
 void default_diagnostic_start_span_fn (const diagnostic_location_print_policy &,
-				       pretty_printer *,
+				       Sink &sink,
 				       expanded_location);
 void default_diagnostic_text_finalizer (diagnostic_text_output_format &,
 					const diagnostic_info *,
