@@ -1,5 +1,5 @@
 /* Print RTL for GCC.
-   Copyright (C) 1987-2024 Free Software Foundation, Inc.
+   Copyright (C) 1987-2026 Free Software Foundation, Inc.
 
 This file is part of GCC.
 
@@ -25,7 +25,6 @@ along with GCC; see the file COPYING3.  If not see
 #include "config.h"
 #endif
 
-#define INCLUDE_MEMORY
 #include "system.h"
 #include "coretypes.h"
 #include "tm.h"
@@ -52,8 +51,13 @@ along with GCC; see the file COPYING3.  If not see
 #include "pretty-print.h"
 #endif
 
+#include "dumpfile.h"
+#include "cfghooks.h"
 #include "print-rtl.h"
 #include "rtl-iter.h"
+#include "json.h"
+
+#include "custom-sarif-properties/cfg.h"
 
 /* Disable warnings about quoting issues in the pp_xxx calls below
    that (intentionally) don't follow GCC diagnostic conventions.  */
@@ -186,8 +190,7 @@ void
 print_mem_expr (FILE *outfile, const_tree expr)
 {
   fputc (' ', outfile);
-  print_generic_expr (outfile, CONST_CAST_TREE (expr),
-		      dump_flags | TDF_SLIM);
+  print_generic_expr (outfile, const_cast<tree> (expr), dump_flags | TDF_SLIM);
 }
 #endif
 
@@ -436,10 +439,10 @@ rtx_writer::print_rtx_operand_codes_E_and_V (const_rtx in_rtx, int idx)
   m_indent -= 2;
 }
 
-/* Subroutine of print_rtx_operand for handling code 'i'.  */
+/* Subroutine of print_rtx_operand for handling code 'L'.  */
 
 void
-rtx_writer::print_rtx_operand_code_i (const_rtx in_rtx, int idx)
+rtx_writer::print_rtx_operand_code_L (const_rtx in_rtx, int idx)
 {
   if (idx == 4 && INSN_P (in_rtx))
     {
@@ -454,10 +457,9 @@ rtx_writer::print_rtx_operand_code_i (const_rtx in_rtx, int idx)
 	  expanded_location xloc = insn_location (in_insn);
 	  fprintf (m_outfile, " \"%s\":%i:%i", xloc.file, xloc.line,
 		   xloc.column);
-	  int discriminator = insn_discriminator (in_insn);
-	    if (discriminator)
+	  if ((dump_flags & TDF_COMPARE_DEBUG) == 0)
+	    if (int discriminator = insn_discriminator (in_insn))
 	      fprintf (m_outfile, " discrim %d", discriminator);
-
 	}
 #endif
     }
@@ -479,7 +481,16 @@ rtx_writer::print_rtx_operand_code_i (const_rtx in_rtx, int idx)
 		 LOCATION_LINE (ASM_INPUT_SOURCE_LOCATION (in_rtx)));
 #endif
     }
-  else if (idx == 5 && NOTE_P (in_rtx))
+  else
+    gcc_unreachable ();
+}
+
+/* Subroutine of print_rtx_operand for handling code 'i'.  */
+
+void
+rtx_writer::print_rtx_operand_code_i (const_rtx in_rtx, int idx)
+{
+  if (idx == 5 && NOTE_P (in_rtx))
     {
       /* This field is only used for NOTE_INSN_DELETED_LABEL, and
 	 other times often contains garbage from INSN->NOTE death.  */
@@ -697,6 +708,10 @@ rtx_writer::print_rtx_operand (const_rtx in_rtx, int idx)
       print_rtx_operand_code_i (in_rtx, idx);
       break;
 
+    case 'L':
+      print_rtx_operand_code_L (in_rtx, idx);
+      break;
+
     case 'p':
       fprintf (m_outfile, " ");
       print_poly_int (m_outfile, SUBREG_BYTE (in_rtx));
@@ -2134,6 +2149,26 @@ rtl_dump_bb_for_graph (pretty_printer *pp, basic_block bb)
     }
 }
 
+
+void
+rtl_dump_bb_as_sarif_properties (diagnostics::sarif_builder *,
+				 json::object &output_bag,
+				 basic_block bb)
+{
+  /* TODO: inter-bb stuff.  */
+  auto json_insn_arr = std::make_unique<json::array> ();
+  rtx_insn *insn;
+  FOR_BB_INSNS (bb, insn)
+    {
+      pretty_printer pp;
+      print_insn_with_notes (&pp, insn);
+      json_insn_arr->append_string (pp_formatted_text (&pp));
+    }
+  output_bag.set_array_of_string
+    (custom_sarif_properties::cfg::basic_block::rtl::insns,
+     std::move (json_insn_arr));
+}
+
 /* Pretty-print pattern X of some insn in non-verbose mode.
    Return a string pointer to the pretty-printer buffer.
 
