@@ -1,5 +1,5 @@
 /* A self-testing framework, for use by -fself-test.
-   Copyright (C) 2015-2024 Free Software Foundation, Inc.
+   Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 This file is part of GCC.
 
@@ -220,10 +220,12 @@ extern void attribs_cc_tests ();
 extern void bitmap_cc_tests ();
 extern void cgraph_cc_tests ();
 extern void convert_cc_tests ();
+extern void dbgcnt_cc_tests ();
 extern void diagnostic_color_cc_tests ();
-extern void diagnostic_format_json_cc_tests ();
+extern void diagnostic_format_html_cc_tests ();
 extern void diagnostic_format_sarif_cc_tests ();
-extern void diagnostic_path_cc_tests ();
+extern void diagnostic_output_spec_cc_tests ();
+extern void diagnostic_path_output_cc_tests ();
 extern void diagnostic_show_locus_cc_tests ();
 extern void digraph_cc_tests ();
 extern void dumpfile_cc_tests ();
@@ -232,29 +234,34 @@ extern void et_forest_cc_tests ();
 extern void fibonacci_heap_cc_tests ();
 extern void fold_const_cc_tests ();
 extern void function_tests_cc_tests ();
+extern void gcc_attribute_urlifier_cc_tests ();
 extern void gcc_urlifier_cc_tests ();
 extern void ggc_tests_cc_tests ();
 extern void gimple_cc_tests ();
+extern void gimple_range_tests ();
+extern void graphviz_cc_tests ();
 extern void hash_map_tests_cc_tests ();
 extern void hash_set_tests_cc_tests ();
 extern void input_cc_tests ();
+extern void ipa_modref_tree_cc_tests ();
 extern void json_cc_tests ();
-extern void lazy_diagnostic_path_cc_tests ();
 extern void json_parser_cc_tests ();
+extern void lazy_diagnostic_path_cc_tests ();
+extern void opt_suggestions_cc_tests ();
 extern void optinfo_emit_json_cc_tests ();
 extern void opts_cc_tests ();
-extern void opts_diagnostic_cc_tests ();
 extern void ordered_hash_map_tests_cc_tests ();
+extern void path_coverage_cc_tests ();
 extern void predict_cc_tests ();
 extern void pretty_print_cc_tests ();
-extern void range_tests ();
 extern void range_op_tests ();
-extern void relation_tests ();
-extern void gimple_range_tests ();
+extern void range_tests ();
 extern void read_rtl_function_cc_tests ();
+extern void relation_tests ();
 extern void rtl_tests_cc_tests ();
 extern void sbitmap_cc_tests ();
 extern void selftest_cc_tests ();
+extern void selftest_logical_location_cc_tests ();
 extern void simple_diagnostic_path_cc_tests ();
 extern void simplify_rtx_cc_tests ();
 extern void spellcheck_cc_tests ();
@@ -269,9 +276,7 @@ extern void typed_splay_tree_cc_tests ();
 extern void vec_cc_tests ();
 extern void vec_perm_indices_cc_tests ();
 extern void wide_int_cc_tests ();
-extern void opt_suggestions_cc_tests ();
-extern void dbgcnt_cc_tests ();
-extern void ipa_modref_tree_cc_tests ();
+extern void xml_cc_tests ();
 
 extern int num_passes;
 
