@@ -13,17 +13,18 @@ foo (unsigned a, unsigned b)
   unsigned _12;
 
   __BB(2):
-  if (a_6(D) > 0u)
+  if (a_6(D) != 0u)
     goto __BB3;
   else
     goto __BB4;
 
   __BB(3):
-  j_10 = __PHI (__BB2: b_7(D));
+  j_10 = __PHI (__BB2: b_11(D));
   _23 = __MIN (a_6(D), j_10);
   goto __BB4;
 
   __BB(4):
   _12 = __PHI (__BB3: _23, __BB2: 0u);
   return _12;
+
 }
